/*
  Copyright 2003 Rikard Bjrklind, Mikael Gransell

  This file is part of dc-qt.

  dc-qt is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  dc-qt is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with dc-qt; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "preferences_dlg.h"
#include "dc_settings.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qvgroupbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlistview.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qfontdialog.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qtabwidget.h>

using namespace std;

preferences_dlg::preferences_dlg(QWidget *p,
			     const char *name,
			     bool modal)
  : QDialog(p, name, modal)
{
     setCaption("dc-qt preferences");
	bool ok;        // used for conversion checking
  
	QTabWidget *tabs = new QTabWidget(this,"muu");
	
	QGridLayout *grid = new QGridLayout(this,4,1);
  
	QHBoxLayout *bottom = new QHBoxLayout;
	grid->addLayout(bottom,3,0);
	QPushButton *okBtn = new QPushButton("&OK",this);
	QPushButton *cancelBtn = new QPushButton("&Cancel",this);
	QPushButton *fontBtn = new QPushButton("Choose &font",this);
	bottom->addWidget(okBtn);
	bottom->addWidget(cancelBtn);
	bottom->addWidget(fontBtn);
	connect(okBtn,SIGNAL(clicked()),this,SLOT(update_settings()));
	connect(okBtn,SIGNAL(clicked()),this,SLOT(close()));
	connect(cancelBtn,SIGNAL(clicked()),this,SLOT(close()));
	connect(fontBtn,SIGNAL(clicked()),this,SLOT(chooseFont()));

//	QGroupBox *userbox = new QGroupBox("User",this,"user box");
	QFrame *userbox = new QFrame(this,"user box");

	QGroupBox *networkbox = new QGroupBox("Network",this,"network box");
	QVGroupBox *dirbox = new QVGroupBox("Sharing",this,"dir box");

	// Name
	QGridLayout *userGrid = new QGridLayout(userbox,3,3,10);
	userGrid->setRowStretch(3,10);
						
	QLabel *editLbl = new QLabel("Name:", userbox);
	nameEdit = new QLineEdit(settings.get_setting(dc_settings::NICKNAME),
							userbox,"nameedit");
	// Description
	QLabel *descLbl = new QLabel("Description:",userbox);
	descEdit = new QLineEdit(settings.get_setting(dc_settings::DESCRIPTION),
							userbox,"descedit");
	// Email
	QLabel *emailLbl = new QLabel("Email:", userbox);
	emailEdit = new QLineEdit(settings.get_setting(dc_settings::EMAIL),userbox,"emailedit");

	userGrid->addWidget(editLbl,0,0);
	userGrid->addWidget(nameEdit,0,1);
	userGrid->addWidget(descLbl,1,0);
	userGrid->addWidget(descEdit,1,1);
	userGrid->addWidget(emailLbl,2,0);
	userGrid->addWidget(emailEdit,2,1);

	// Network section

	QGridLayout *netGrid = new QGridLayout(networkbox,12,4,14);
	
	// Port
	QLabel *portLbl = new QLabel("Port:",networkbox);
	int portnr = settings.get_setting(dc_settings::PORT).toInt(&ok);
	portSpin = new QSpinBox(0,65535,1,networkbox);
	portSpin->setValue(portnr);
	
	// Slots
	QLabel *slotsLbl = new QLabel("Upload slots:",networkbox);
	int slotnr = settings.get_setting(dc_settings::SLOTS).toInt(&ok);
	slotsSpin = new QSpinBox(1,15,1,networkbox);
	slotsSpin->setValue(slotnr);

	// Max upload speed
	QLabel *speedLbl = new QLabel("Max upload speed (kb/s):",networkbox);
	int limit = settings.get_setting(dc_settings::LIMIT).toInt(&ok);
	speedSpin = new QSpinBox(0,99999,1,networkbox);
	speedSpin->setValue(limit);

	// Address
	QLabel *addrLbl = new QLabel("Forced IP (leave blank for default):",networkbox);
	addrEdit = new QLineEdit(settings.get_setting(dc_settings::ADDRESS),
			   networkbox,"Address");

	// Connection type, sorry for the ugly code.
	connectionCombo = new QComboBox(networkbox,"connectiontype");
	QStringList connTypes = settings.get_connection_types();
	int connTypeIndex = connTypes.findIndex(settings.get_setting(dc_settings::CONNECTION));
	connectionCombo->insertStringList( connTypes );
	connectionCombo->setCurrentItem(connTypeIndex);
  
	// passive
	passive = new QCheckBox("Passive connection",networkbox);
	if(settings.get_setting(dc_settings::PASSIVE) == "true")
		passive->setChecked(true);

	// Proxy stuff
	proxyCheck = new QCheckBox("Use proxy",networkbox);
	if(settings.get_setting(dc_settings::USEPROXY)=="true") proxyCheck->setChecked(true);
	QLabel *proxyAddrLbl = new QLabel("Proxy address:",networkbox);
	proxyAddr = new QLineEdit(settings.get_setting(dc_settings::PROXYADDRESS),networkbox,"proxyaddress");
	QLabel *proxyPortLbl = new QLabel("Port:",networkbox);
	proxyPort = new QLineEdit(settings.get_setting(dc_settings::PROXYPORT),networkbox,"proxyport");
	if(proxyPort->text() == QString::null)
	     proxyPort->setText("8080");
	proxyUseSocks = new QCheckBox("Use SOCKS",networkbox);
	if(settings.get_setting(dc_settings::USESOCKS)=="true") proxyUseSocks->setChecked(true);
	QLabel *proxyUserLbl = new QLabel("Username:",networkbox);
	proxyUsername = new QLineEdit(settings.get_setting(dc_settings::PROXYUSERNAME),networkbox,"proxyusername");
	QLabel *proxyPwdLbl = new QLabel("Password:",networkbox);
	proxyPassword = new QLineEdit(settings.get_setting(dc_settings::PROXYPWD),networkbox,"proxypassword");
	
	if(!proxyCheck->isChecked())
	     disableProxyWidgets(!true);
	if(!proxyUseSocks->isChecked())
	     disableSocksWidgets(!true);

	connect(proxyCheck,SIGNAL(toggled(bool)),this,SLOT(disableProxyWidgets(bool)));
	connect(proxyUseSocks,SIGNAL(toggled(bool)),this,SLOT(disableSocksWidgets(bool)));

	netGrid->addWidget(portLbl,0,0);
	netGrid->addWidget(portSpin,0,1);
	netGrid->addWidget(slotsLbl,1,0);
	netGrid->addWidget(slotsSpin,1,1);
	netGrid->addWidget(speedLbl,2,0);
	netGrid->addWidget(speedSpin,2,1);
	netGrid->addWidget(addrLbl,3,0);
	netGrid->addWidget(addrEdit,3,1);
	netGrid->addWidget(connectionCombo,4,1);
	netGrid->addWidget(passive,5,1);
	netGrid->addWidget(proxyCheck,6,0);
	netGrid->addWidget(proxyAddrLbl,7,0);
	netGrid->addWidget(proxyAddr,7,1);
	netGrid->addWidget(proxyPortLbl,8,0);
	netGrid->addWidget(proxyPort,8,1);
	netGrid->addWidget(proxyUseSocks,9,0);
	netGrid->addWidget(proxyUserLbl,10,0);
	netGrid->addWidget(proxyUsername,10,1);
	netGrid->addWidget(proxyPwdLbl,11,0);
	netGrid->addWidget(proxyPassword,11,1);
			   

	// Folders section TODO DnD
	// shared
	shared = new QListView(dirbox,"sharedlist");
	shared->addColumn("Directory");
  
	for(int i=0;i < settings.num_shares();i++)
		QListViewItem *directory = new QListViewItem(shared,settings.get_share(i));

	QHBox *hbox1 = new QHBox(dirbox);

	QPushButton *add = new QPushButton("add",hbox1);
	QPushButton *remove = new QPushButton("remove",hbox1);

	connect(add,SIGNAL(clicked()),this,SLOT(addSharesDlg()));
	connect(remove,SIGNAL(clicked()),this,SLOT(removeSelectedShare()));

	// DL path
	QHBox *hbox2 = new QHBox(dirbox);
	QLabel *dlLbl = new QLabel("Download path:",hbox2);
	dlEdit = new QLineEdit(settings.get_setting(dc_settings::DLDIR),hbox2,"dlpath"); 

	QPushButton *dlBrowseBtn = new QPushButton("...",hbox2);
	connect(dlBrowseBtn,SIGNAL(clicked()),this,SLOT(openDirSelDlg()));

	tabs->addTab(networkbox,"Connection");
	tabs->addTab(userbox,"User");
	tabs->addTab(dirbox,"Directories");
	grid->addWidget(tabs,0,0);	
}

void preferences_dlg::openDirSelDlg()
{
	QString dir = QFileDialog::getExistingDirectory(settings.get_home_dir(), this, "dldir_dialog",
													"Select download directory");
	dlEdit->setText(dir);
}

void preferences_dlg::removeSelectedShare()
{
	shared->takeItem( shared->selectedItem() );
}

void preferences_dlg::addSharesDlg()
{
	QString dir = QFileDialog::getExistingDirectory(settings.get_home_dir(), this, "shdir_dialog",
													"Select a sharing directory");

	if(dir != QString::null)
		QListViewItem *item = new QListViewItem(shared,dir);
}

void preferences_dlg::chooseFont()
{
	bool ok;
	font = QFontDialog::getFont( &ok, QApplication::font(), this );
	if ( ok ) {
	// Set tahe desired font
	QApplication::setFont(font,true);
	
	} else {
	// Font dialog was cancelled.
    }
}

//  update_settings()
//   Typically called when someone presses OK.
//   writes all config values to the preferences object,
void preferences_dlg::update_settings()
{
	settings.set_setting( dc_settings::NICKNAME, nameEdit->text() );
	settings.set_setting( dc_settings::DESCRIPTION, descEdit->text() );
	settings.set_setting( dc_settings::EMAIL, emailEdit->text() );
	settings.set_setting( dc_settings::ADDRESS, addrEdit->text() );
	settings.set_setting( dc_settings::SLOTS, slotsSpin->text() );
	settings.set_setting( dc_settings::PORT, portSpin->text() );
	settings.set_setting( dc_settings::CONNECTION, connectionCombo->currentText() );
	settings.set_setting( dc_settings::PASSIVE, passive->isChecked() ? "true" : "false");
	settings.set_setting( dc_settings::LIMIT, speedSpin->text() );
	settings.set_setting( dc_settings::DLDIR, dlEdit->text() );
	settings.set_setting( dc_settings::FONT, font.toString() );

	settings.set_setting( dc_settings::PROXYADDRESS, proxyAddr->text() );
	settings.set_setting( dc_settings::PROXYPORT, proxyPort->text() );
	settings.set_setting( dc_settings::PROXYUSERNAME, proxyUsername->text() );
	settings.set_setting( dc_settings::PROXYPWD, proxyPassword->text() );
	settings.set_setting( dc_settings::USEPROXY, proxyCheck->isChecked() ? "true" : "false");
	settings.set_setting( dc_settings::USESOCKS, proxyUseSocks->isChecked() ? "true" : "false");


	// Add shares
	settings.clear_shares();
	QListViewItem *item = shared->firstChild();
	while(item) 
	{
		settings.add_share(item->text(0));
		item = item->nextSibling();
	}

#ifdef _DEBUG
	settings.print();
#endif

	if(!settings.save_to_file())
	     QMessageBox::critical(this, "Error", "Error saving the preferences file.","Ok");
}

void preferences_dlg::disableProxyWidgets(bool yes)
{
     proxyAddr->setDisabled(!yes);
     proxyPort->setDisabled(!yes);
     proxyUseSocks->setDisabled(!yes);
     if(yes)
	  disableSocksWidgets(proxyUseSocks->isChecked());
     else disableSocksWidgets(false);

}

void preferences_dlg::disableSocksWidgets(bool yes)
{
     proxyUsername->setDisabled(!yes);
     proxyPassword->setDisabled(!yes);
}
